/****************************************************************************
**
** Copyright (C) 2020 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtQml module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**/
#ifndef QQMLDOM_FWD_P_H
#define QQMLDOM_FWD_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include "qqmldom_global.h"

QT_BEGIN_NAMESPACE

namespace QQmlJS {
namespace Dom {

class AstComments;
class AttachedInfo;
class Binding;
class Comment;
class CommentedElement;
class ConstantData;
class DomBase;
class DomEnvironment;
class DomItem;
class DomTop;
class DomUniverse;
class Empty;
class EnumDecl;
class Export;
class ExternalItemInfoBase;
class ExternalItemPairBase;
class ExternalOwningItem;
class FileLocations;
class FileWriter;
class GlobalComponent;
class GlobalScope;
class MockObject;
class MockOwner;
class Id;
class Import;
class JsFile;
class JsResource;
class List;
class LoadInfo;
class Map;
class MethodInfo;
class ModuleIndex;
class ModuleScope;
class MutableDomItem;
class ObserversTrie;
class OutWriter;
class OutWriterState;
class OwningItem;
class Path;
class Pragma;
class PropertyDefinition;
class PropertyInfo;
class QmlDomAstCreator;
class QmlComponent;
class QmlDirectory;
class QmldirFile;
class QmlFile;
class QmlObject;
class QmltypesComponent;
class QmltypesFile;
class Reference;
class RegionComments;
class ScriptExpression;
class Source;
class TestDomItem;
class Version;

} // end namespace Dom
} // end namespace QQmlJS
QT_END_NAMESPACE
#endif // QQMLDOM_FWD_P_H
